from jasmine.classes_and_files_reader.datachallenge_lightcurve_cls import LightcurveEventDataChallenge
import pandas as pd
import matplotlib.pyplot as plt


list_of_bound_planet_events = [4, 8, 12, 25, 32, 40, 47, 50, 53, 62, 66, 69, 74, 78, 81, 92, 95, 99, 100, 103,
                               107, 124, 128, 131, 139, 152, 163, 186, 193, 194, 199, 208, 214, 217, 218, 223,
                               226, 227, 250, 253, 258, 267, 289]
# list_of_bound_planet_events = [4,]
master_path = '/local/data/emussd1/greg_shared/rtmodel_effort/datachallenge'
# master_path = '/Users/sishitan/Documents/Scripts/jasmine'
df_general = []
for event_number in list_of_bound_planet_events:
    true_values_path = f'{master_path}/data'
    the_lightcurve_event = LightcurveEventDataChallenge(event_number, true_values_path)
    true_q = the_lightcurve_event.planet.planet_mass_ratio
    true_s = the_lightcurve_event.planet.planet_separation
    data_to_be_saved = {'data_challenge_lc_number': [event_number,],
                        'true_q': [true_q,],
                        'true_s': [true_s,],}
    df_for_one_event = pd.DataFrame(data_to_be_saved)
    df_general.append(df_for_one_event)
# Concatenate all the dataframes
wrapper_df = pd.concat(df_general)
wrapper_df.plot(x='true_s', y='true_q', kind='scatter', title='q VS s: 43 planet events', alpha=0.5)
plt.savefig('q_vs_s_43_planet_events.png', dpi=300)

wrapper_df.plot(x='true_s', y='true_q', kind='scatter', title='q VS s: 43 planet events', alpha=0.5)
plt.xscale('log')
plt.yscale('log')
plt.savefig('log_scale_q_vs_s_43_planet_events.png', dpi=300)



def process_file(input_file, output_file, *,
                 new_mass_ratio=None,
                 new_separation=None,
                 new_rho=None):
    with open(input_file, 'r') as infile, open(output_file, 'w') as outfile:
        for line in infile:
            stripped_line = line.strip()
            if stripped_line:
                values = stripped_line.split()
                # Write the original line
                outfile.write('\t'.join(values) + '\n')
                if new_mass_ratio is not None:
                    if len(values) > 1:
                        values[1] = new_mass_ratio
                        outfile.write('\t'.join(values) + '\n')
                if new_separation is not None:
                    if len(values) > 1:
                        values[0] = new_separation
                        outfile.write('\t'.join(values) + '\n')
                if new_rho is not None:
                    if len(values) > 1:
                        values[4] = new_separation
                        outfile.write('\t'.join(values) + '\n')



root_path = '/Users/stela/Documents/Scripts/RTModel_project/RTModel/RTModel/data/'
input_file_path = root_path + 'TemplateLibrary.txt'
output_file_path = root_path + 'TemplateLibrary_SIS.txt'
# Run the function
process_file(input_file_path, output_file_path, new_mass_ratio='0.0001')


import pandas as pd


def binary_star_master_reader(filepath_='data/binary_star.csv'):
    """
    This function reads the binary star file and returns a pandas dataframe with the columns
    :param filepath_:
    :return: binary_star_df
    """
    binary_star_df = pd.read_csv(filepath_)
    return binary_star_df


def bound_planet_master_reader(filepath_='data/bound_planet.csv'):
    """
    This function reads the bound planet file and returns a pandas dataframe with the columns
    :param filepath_:
    :return: bound_planet_df
    """
    bound_planet_df = pd.read_csv(filepath_)
    return bound_planet_df


def cataclysmic_variables_master_reader(filepath_='data/cataclysmic_variables.csv'):
    """
    This function reads the cataclysmic variables file and returns a pandas dataframe with the columns
    :param filepath_:
    :return: cataclysmic_variables_df
    """
    cataclysmic_variables_df = pd.read_csv(filepath_)
    return cataclysmic_variables_df


def single_lens_master_reader(filepath_='data/single_lens.csv'):
    """
    This function reads the single lens file and returns a pandas dataframe with the columns
    :param filepath_:
    :return: single_lens_df
    """
    single_lens_df = pd.read_csv(filepath_)
    return single_lens_df


def what_type_of_lightcurve(data_challenge_lc_number_, *,
                            binary_star_path_='data/binary_star.csv',
                            bound_planet_path_='data/bound_planet.csv',
                            cataclysmic_variables_path_='data/cataclysmic_variables.csv',
                            single_lens_path_='data/single_lens.csv'):
    """
    This function reads the master files and checks in which file the lightcurve is.
    :param single_lens_path_:
    :param cataclysmic_variables_path_:
    :param bound_planet_path_:
    :param binary_star_path_:
    :param data_challenge_lc_number_:
    :return: master_name
    """
    binary_star_master_df = binary_star_master_reader(binary_star_path_)
    bound_planet_master_df = bound_planet_master_reader(bound_planet_path_)
    cataclysmic_variables_master_df = cataclysmic_variables_master_reader(cataclysmic_variables_path_)
    single_lens_master_df = single_lens_master_reader(single_lens_path_)
    masters_dict = {'binary_star': binary_star_master_df,
                    'bound_planet': bound_planet_master_df,
                    'cataclysmic_variables': cataclysmic_variables_master_df,
                    'single_lens': single_lens_master_df}

    for master_name, dataframe in masters_dict.items():
        LC_check = dataframe[dataframe['data_challenge_lc_number'] == data_challenge_lc_number_]['data_challenge_lc_number'].values
        if LC_check.size > 0:
            print(f'Ligthcurve {data_challenge_lc_number_} found on {master_name}.csv file.')
            return master_name

    raise ValueError(f'Lightcurve {data_challenge_lc_number_} not found in any of the master files.')


def lightcurve_data_reader(data_challenge_lc_number_, *, filter_='W149', folder_path_='data'):
    """
    This function reads the lightcurve data file and returns a pandas dataframe with the BJD Magnitude Error columns
    :param folder_path_:
    :param data_challenge_lc_number_:
    :param filter_:
    :return: lightcurve_data_df
    """
    lightcurve_data_df = pd.read_csv(f'{folder_path_}/ulwdc1_{data_challenge_lc_number_:03}_{filter_}.txt',
                                     names=['BJD', 'Magnitude', 'Error'], sep=' ')
    lightcurve_data_df['days'] = lightcurve_data_df['BJD'] - 2450000
    return lightcurve_data_df


if __name__ == '__main__':
    lc_type = what_type_of_lightcurve(2)
    print(lc_type)




from dataclasses import dataclass

import jasmine.classes_and_files_reader.datachallenge_reader as dcr
import pandas as pd


class LightcurveEventDataChallenge:
    def __init__(self, data_challenge_lc_number, data_folder='../data'):
        self.lc_number = data_challenge_lc_number
        self.data_folder = data_folder
        self.lc_type = dcr.what_type_of_lightcurve(data_challenge_lc_number,
                                                   binary_star_path_=f'{data_folder}/binary_star.csv',
                                                   bound_planet_path_=f'{data_folder}/bound_planet.csv',
                                                   cataclysmic_variables_path_=f'{data_folder}/cataclysmic_variables.csv',
                                                   single_lens_path_=f'{data_folder}/single_lens.csv'
                                                   )
        master_path = f'{data_folder}/{self.lc_type}.csv'
        master_df = pd.read_csv(master_path)
        self.lightcurve_master = master_df[master_df['data_challenge_lc_number'] == self.lc_number]
        self.event_info = EventInformation(self.lightcurve_master)
        self.source = SourceStarProperties(self.lightcurve_master)
        self.weights_and_flags = WeightAndFlags(self.lightcurve_master, self.lc_type)
        if self.lc_type == 'bound_planet' or self.lc_type == 'binary_star' or self.lc_type == 'single_lens':
            self.lens = LensStarProperties(self.lightcurve_master)
            self.microlensing = MicrolensingParameters(self.lightcurve_master)
        if self.lc_type == 'bound_planet':
            self.planet = PlanetProperties(self.lightcurve_master)
        if self.lc_type == 'binary_star':
            self.second_lens = SecondLens(self.lightcurve_master)

    def lightcurve_data(self, *, filter_, folder_path_='../data'):
        """
        This function reads the lightcurve data and returns a pandas dataframe with the columns
        :param filter_:
        :param folder_path_:
        :return:
        """
        lightcurve_df = dcr.lightcurve_data_reader(self.lc_number, filter_=filter_, folder_path_=folder_path_)
        return lightcurve_df


@dataclass
class EventInformation:
    """
    Field and Coordinate Information: This includes the event's index, subrun number, field,
    galactic coordinates (l, b), and right ascension and declination (ra, dec).
    """
    event_index: int
    event_subrun: int
    event_field: int
    event_galactic_longitude__deg: float
    event_galactic_latitude__deg: float
    event_right_ascension__deg: float
    event_declination__deg: float
    event_type: str
    lc_root: str
    data_challenge_lc_number: int

    def __init__(self, lightcurve_master: pd.DataFrame):
        self.event_index = lightcurve_master['idx'].values[0]
        self.event_subrun = lightcurve_master['subrun'].values[0]
        self.event_field = lightcurve_master['field'].values[0]
        self.event_galactic_longitude__deg = lightcurve_master['l'].values[0]
        self.event_galactic_latitude__deg = lightcurve_master['b'].values[0]
        self.event_right_ascension__deg = lightcurve_master['ra'].values[0]
        self.event_declination__deg = lightcurve_master['dec'].values[0]
        self.event_type = lightcurve_master['event_type'].values[0]
        self.lc_root = lightcurve_master['lc_root'].values[0]
        self.data_challenge_lc_number = lightcurve_master['data_challenge_lc_number'].values[0]


@dataclass
class SourceStarProperties:
    """
    Source Star Properties: Details about the source star, such as its ID, distance (Ds), radius (Rs),
    proper motion (smu_l, smu_b), age, class (scl), and type (styp).
    Magnitudes: The magnitudes of the source in various filters (e.g., J, F087, H, W149, W169).
    """
    source_id: int
    source_distance__kpc: float
    source_radius__rsun: float
    source_age: int
    source_class: int
    source_type: float
    source_proper_motion_l_1masyr: float
    source_proper_motion_b_1masyr: float
    source_magnitude_J__mag: float
    source_magnitude_F087__mag: float
    source_magnitude_H__mag: float
    source_magnitude_W149__mag: float
    source_magnitude_W169__mag: float

    def __init__(self, lightcurve_master: pd.DataFrame):
        self.source_id = lightcurve_master['src_id'].values[0]
        self.source_distance__kpc = lightcurve_master['Ds'].values[0]
        self.source_radius__rsun = lightcurve_master['Rs'].values[0]
        self.source_age = lightcurve_master['sage'].values[0]
        self.source_class = lightcurve_master['scl'].values[0]
        self.source_type = lightcurve_master['styp'].values[0]

        self.source_proper_motion_l_1masyr = lightcurve_master['smu_l'].values[0]
        self.source_proper_motion_b_1masyr = lightcurve_master['smu_b'].values[0]

        self.source_magnitude_J__mag = lightcurve_master['Js'].values[0]
        self.source_magnitude_F087__mag = lightcurve_master['F087s'].values[0]
        self.source_magnitude_H__mag = lightcurve_master['Hs'].values[0]
        self.source_magnitude_W149__mag = lightcurve_master['W149s'].values[0]
        self.source_magnitude_W169__mag = lightcurve_master['W169s'].values[0]


@dataclass
class LensStarProperties:
    """
    Lens Star Properties: Information on the lens star, including its ID, distance (Dl), mass (Ml),
    proper motion (lmu_l, lmu_b), age, class (lcl), type (ltyp), bolometric magnitude (lMbol),
    effective temperature (lTeff), surface gravity (Llogg), and radius (Rl).
    """
    lens_system_mass__msun: float
    lens_distance__kpc: float
    lens_radius__rsun: float
    lens_proper_motion_l_1masyr: float
    lens_proper_motion_b_1masyr: float
    lens_id: int
    lens_age: int
    lens_class: int
    lens_type: float
    lens_surface_gravity: float
    lens_bolometric_magnitude: float
    lens_effective_temperature__k: float
    lens_magnitude_J__mag: float
    lens_magnitude_F087__mag: float
    lens_magnitude_H__mag: float
    lens_magnitude_W149__mag: float
    lens_magnitude_W169__mag: float

    def __init__(self, lightcurve_master: pd.DataFrame):
        # MASSES
        # Ml: mass of the lens
        self.lens_system_mass__msun = lightcurve_master['Ml'].values[0]

        # Dl: distance to the lens
        self.lens_distance__kpc = lightcurve_master['Dl'].values[0]
        #Radius
        self.lens_radius__rsun = lightcurve_master['Rl'].values[0]
        # Motion
        self.lens_proper_motion_l_1masyr = lightcurve_master['lmu_l'].values[0]
        self.lens_proper_motion_b_1masyr = lightcurve_master['lmu_b'].values[0]

        # Other lens parameters
        self.lens_id = lightcurve_master['lens_id'].values[0]
        self.lens_age = lightcurve_master['lage'].values[0]
        self.lens_class = lightcurve_master['lcl'].values[0]
        self.lens_type = lightcurve_master['ltyp'].values[0]

        self.lens_surface_gravity = lightcurve_master['Llogg'].values[0]
        self.lens_bolometric_magnitude = lightcurve_master['lMbol'].values[0]
        self.lens_effective_temperature__k = lightcurve_master['lTeff'].values[0]

        # Magnitudes: The magnitudes of the lens in various filters (e.g., J, F087, H, W149, W169).
        self.lens_magnitude_J__mag = lightcurve_master['Jl'].values[0]
        self.lens_magnitude_F087__mag = lightcurve_master['F087l'].values[0]
        self.lens_magnitude_H__mag = lightcurve_master['Hl'].values[0]
        self.lens_magnitude_W149__mag = lightcurve_master['W149l'].values[0]
        self.lens_magnitude_W169__mag = lightcurve_master['W169l'].values[0]


@dataclass
class MicrolensingParameters:
    """
    Microlensing Event Properties: Parameters such as the impact parameter (u0), event angle (alpha),
    time of maximum magnification (t0), Einstein crossing time (tE), Einstein radius (rE), theta_E (thE),
    parallax effect (piE), source radius in Einstein radii (rhos), relative proper motion (murel),
    transverse velocity (vt), and shear (gamma).

    Blending Flux (fs): The blending parameter, which is crucial for determining the combined
    flux from the source, lens, and any additional blended light.
    """
    microlensing_impact_parameter_u0: float
    microlensing_event_angle_alpha__deg: float
    microlensing_time_of_peak_t0__days: float
    microlensing_einstein_crossing_time_tE__days: float
    microlensing_einstein_radius_rE__AU: float
    microlensing_einstein_angular_radius_thetaE__mas: float
    microlensing_parallax_effect_piE: float
    microlensing_source_radius_in_einstein_radii_rho: float
    microlensing_relative_proper_motion_murel__1masyr: float
    microlensing_transverse_velocity_vt__km1s: float
    microlensing_shear_gamma: float
    microlensing_blending_flux_fs0: float
    microlensing_blending_flux_fs1: float

    def __init__(self, lightcurve_master: pd.DataFrame):
        self.microlensing_impact_parameter_u0 = lightcurve_master['u0'].values[0]
        self.microlensing_event_angle_alpha__deg = lightcurve_master['alpha'].values[0]
        self.microlensing_time_of_peak_t0__days = lightcurve_master['t0'].values[0]
        self.microlensing_einstein_crossing_time_tE__days = lightcurve_master['tE'].values[0]
        self.microlensing_einstein_radius_rE__AU = lightcurve_master['rE'].values[0]
        self.microlensing_einstein_angular_radius_thetaE__mas = lightcurve_master['thE'].values[0]
        self.microlensing_parallax_effect_piE = lightcurve_master['piE'].values[0]
        self.microlensing_source_radius_in_einstein_radii_rho = lightcurve_master['rhos'].values[0]
        self.microlensing_relative_proper_motion_murel__1masyr = lightcurve_master['murel'].values[0]
        self.microlensing_transverse_velocity_vt__km1s = lightcurve_master['vt'].values[0]
        self.microlensing_shear_gamma = lightcurve_master['gamma'].values[0]
        self.microlensing_blending_flux_fs0 = lightcurve_master['fs0'].values[0]
        self.microlensing_blending_flux_fs1 = lightcurve_master['fs1'].values[0]


@dataclass
class WeightAndFlags:
    """
    Weight and Flags: Includes the raw weight (raww), final weight (w), and flags indicating
    certain conditions of the simulation (e.g., FSflag, flatsatFlag).
    Analysis Flags and Chi-square Values: Indicators for the analysis status, including
    flat lightcurve flag (flatchi2), chi-square values for different fits (chi2_0, chi2_1, chi2),
    and the normalization weight (normw).
    """
    weight_and_flags_u0max: float
    weight_and_flags_raw_weight: float
    weight_and_flags_final_weight: float
    weight_and_flags_FS_flag: int
    weight_and_flags_flatsat_flag: int
    weight_and_flags_flat_lightcurve_flag: int
    weight_and_flags_chi2_0: float
    weight_and_flags_chi2_1: int
    weight_and_flags_chi2: float
    weight_and_flags_normalization_weight: float

    def __init__(self, lightcurve_master: pd.DataFrame, lc_type: str):
        self.weight_and_flags_u0max = lightcurve_master['u0max'].values[0]
        self.weight_and_flags_raw_weight = lightcurve_master['raww'].values[0]
        self.weight_and_flags_final_weight = lightcurve_master['w'].values[0]
        self.weight_and_flags_FS_flag = lightcurve_master['FSflag'].values[0]
        self.weight_and_flags_flatsat_flag = lightcurve_master['flatsatFlag'].values[0]

        self.weight_and_flags_flat_lightcurve_flag = lightcurve_master['flatchi2'].values[0]
        self.weight_and_flags_chi2_0 = lightcurve_master['chi2_0'].values[0]
        self.weight_and_flags_chi2_1 = lightcurve_master['chi2_1'].values[0]
        self.weight_and_flags_chi2 = lightcurve_master['chi2'].values[0]

        #if lc_type in ['bound_planet', 'binary_star', 'single_lens']:
        #    self.weight_and_flags_normalization_weight = lightcurve_master['normw'].values[0]


@dataclass
class PlanetProperties:
    """
    Planet Properties: For simulations involving planets,
    this section lists the planet's mass (Mp),
    semi-major axis (a), inclination (inc), orbital phase (phase), mass ratio to the host star (q),
    separation (s), and orbital period (period).
    """
    planet_mass_ratio: float
    planet_mass__msun: float
    lens_system_mass__msun: float
    lens_planet_mass_ver: float
    lens_host_mass__msun: float
    planet_semi_major_axis__au: float
    planet_separation: float
    planet_inclination__deg: float
    planet_orbital_phase: float
    planet_orbital_period__years: float

    def __init__(self, lightcurve_master: pd.DataFrame):
        # q: mass ratio
        self.planet_mass_ratio = lightcurve_master['q'].values[0]
        # Mp: mass of the planet
        self.planet_mass__msun = lightcurve_master['Mp'].values[0]
        # mass of the planet calculated from the mass ratio
        # Ml: mass of the lens
        self.lens_system_mass__msun = lightcurve_master['Ml'].values[0]

        self.lens_planet_mass_ver = (self.planet_mass_ratio / (
                    self.planet_mass_ratio + 1)) * self.lens_system_mass__msun
        # mass of the host star
        self.lens_host_mass__msun = self.lens_system_mass__msun / (1 + self.planet_mass_ratio)

        # Distances
        # a: semi-major axis
        self.planet_semi_major_axis__au = lightcurve_master['a'].values[0]
        # separation
        self.planet_separation = lightcurve_master['s'].values[0]
        self.planet_inclination__deg = lightcurve_master['inc'].values[0]
        self.planet_orbital_phase = lightcurve_master['phase'].values[0]
        # period
        self.planet_orbital_period__years = lightcurve_master['period'].values[0]


@dataclass
class SecondLens:
    """
    Second lens Properties: For simulations involving binary stars,
    this section lists the secondary mass (Mp),
    semi-major axis (a), inclination (inc), orbital phase (phase), mass ratio to the host star (q),
    separation (s), and orbital period (period).
    """
    second_lens_mass_ratio: float
    second_lens_mass__msun: float
    lens_system_mass__msun: float
    lens_planet_mass_ver: float
    lens_host_mass__msun: float
    second_lens_semi_major_axis__au: float
    second_lens_separation: float
    second_lens_inclination__deg: float
    second_lens_orbital_phase: float
    second_lens_orbital_period__years: float

    def __init__(self, lightcurve_master: pd.DataFrame):
        # q: mass ratio
        self.second_lens_mass_ratio = lightcurve_master['q'].values[0]
        # Mp: mass of the planet
        self.second_lens_mass__msun = lightcurve_master['Mp'].values[0]
        # mass of the planet calculated from the mass ratio
        # Ml: mass of the lens
        self.lens_system_mass__msun = lightcurve_master['Ml'].values[0]

        self.lens_planet_mass_ver = (self.second_lens_mass_ratio / (
                    self.second_lens_mass_ratio + 1)) * self.lens_system_mass__msun
        # mass of the host star
        self.lens_host_mass__msun = self.lens_system_mass__msun / (1 + self.second_lens_mass_ratio)

        # Distances
        # a: semi-major axis
        self.second_lens_semi_major_axis__au = lightcurve_master['a'].values[0]
        # separation
        self.second_lens_separation = lightcurve_master['s'].values[0]
        self.second_lens_inclination__deg = lightcurve_master['inc'].values[0]
        self.second_lens_orbital_phase = lightcurve_master['phase'].values[0]
        # period
        self.second_lens_orbital_period__years = lightcurve_master['period'].values[0]




from jasmine import LightcurveEventDataChallenge
import numpy as np
import VBBinaryLensing


def datachallenge_bound_planet_magnification_using_vbb(lightcurve_, *,
                                                       time_interval=None,
                                                       parallax=False,
                                                       orbital_motion=False):
    """
    Note that parallax is not added yet
    TODO: calculate the parallax and OM
    :param lightcurve_:
    :param time_interval:
    :param parallax:
    :param orbital_motion:
    :return:
    """

    vbb = VBBinaryLensing.VBBinaryLensing()

    separation_s = lightcurve_.planet.planet_separation
    mass_ratio_q = lightcurve_.planet.planet_mass_ratio
    alpha = lightcurve_.microlensing.microlensing_event_angle_alpha__deg
    impact_parameter_u0 = lightcurve_.microlensing.microlensing_impact_parameter_u0
    einstein_time_tE = lightcurve_.microlensing.microlensing_einstein_crossing_time_tE__days
    peak_time_t0 = lightcurve_.microlensing.microlensing_time_of_peak_t0__days
    rho = lightcurve_.microlensing.microlensing_source_radius_in_einstein_radii_rho
    if time_interval is None:
        time_interval = np.linspace(peak_time_t0 - 2 * einstein_time_tE,
                                    peak_time_t0 + 2 * einstein_time_tE,
                                    100000)
    if parallax:
        print('Edit here to a collection of parameters with parallax')
    if orbital_motion:
        print('Edit here to a collection of parameters with orbital motion')
    pr = [np.log(separation_s), np.log(mass_ratio_q), impact_parameter_u0,
          alpha, np.log(rho), np.log(einstein_time_tE), peak_time_t0]
    results = vbb.BinaryLightCurve(pr, time_interval)

    return results[0], time_interval


def easy_plot_lightcurve(lightcurve_):
    import matplotlib.pyplot as plt
    fig, ax = plt.subplots()
    magnification, time_interval = datachallenge_bound_planet_magnification_using_vbb(lightcurve_)
    ax.scatter(time_interval, magnification, s=0.8, alpha=0.3)
    plt.show(dpi=600)


if __name__ == '__main__':
    template_path = '/Users/stela/Documents/Scripts/RTModel_project/RTModel/RTModel/data/TemplateLibrary.txt'
    lightcurve = LightcurveEventDataChallenge(data_challenge_lc_number=267,
                                              data_folder='/Users/stela/Documents/Scripts/RTModel_project/datachallenge')

    datachallenge_bound_planet_magnification_using_vbb(lightcurve)
    easy_plot_lightcurve(lightcurve)




from jasmine import RTModelTemplateForBinaryLightCurve
import numpy as np
import pandas as pd

def easy_plot_lightcurve(rtmodel_template_two_lenses_):
    import matplotlib.pyplot as plt
    fig, ax = plt.subplots()
    magnification, time_interval = rtmodel_template_two_lenses_.rtmodel_magnification_using_vbb()
    ax.scatter(time_interval, magnification, s=0.8, alpha=0.3)
    plt.show(dpi=600)


def all_rtmodel_template_generator(template_path, output_folder):
    '''
    SIS: I accidentally committed this part and this function is not ready yet.
    DO NOT USE IT
    '''
    lines = np.arange(2, 114)
    times_peak = np.arange(1, 402, 5)
    times_peak_2 = np.arange(1, 402, 2)
    base_times_peak_2 = np.arange(1, 402, 4)
    shuffled_peak_2 = np.random.shuffle(base_times_peak_2)
    times_peak_2 = base_times_peak_2.append(shuffled_peak_2)

    # intervals =
    for line in lines:
        rtmodel_template_two_lenses = RTModelTemplateForBinaryLightCurve(template_line=line,
                                                                         path_to_template=template_path,
                                                                         input_peak_t1=300,
                                                                         input_peak_t2=302)
        magnification, times = rtmodel_template_two_lenses.rtmodel_magnification_using_vbb()
        binary_lens_signal_df = pd.DataFrame({'Magnification': magnification,
                                              'times': times})
        binary_lens_signal_df.to_csv(f'{output_folder}_rtmodel_{line}.csv')


if __name__ == '__main__':
    template_path = '/Users/stela/Documents/Scripts/RTModel_project/RTModel/RTModel/data/TemplateLibrary.txt'
    rtmodel_template_two_lenses = RTModelTemplateForBinaryLightCurve(template_line=2,
                                                                     path_to_template=template_path,
                                                                     input_peak_t1=300,
                                                                     input_peak_t2=302)
    # easy_plot_lightcurve(rtmodel_template_two_lenses)




import glob
import pandas as pd

from jasmine.classes_and_files_reader.datachallenge_lightcurve_cls import LightcurveEventDataChallenge
from jasmine.classes_and_files_reader.RTModel_results_cls import ModelResults


def chi2_getter(filepath):
    """
    This function reads the chi2 file and returns the chi2 value
    :param filepath:
    :return: chi2
    """
    with open(filepath, 'r') as f:
        lines = f.readlines()
        chi2 = float(lines[0].split(' ')[-1])
    return chi2


def models_per_chi2_rank(folder_path):
    """
    This function rank the models per chi2 value
    """
    models = glob.glob(folder_path +'/Models/*txt')
    models_names = [model.replace(folder_path, '').replace('/Models/', '').replace('.txt', '') for model in models]
    chi2_values = []
    for model in models:
        chi2_values.append(chi2_getter(model))
    chi2_df = pd.DataFrame({'model': models_names, 'chi2': chi2_values})
    chi2_df = chi2_df.sort_values(by='chi2')
    chi2_df.to_csv(folder_path + '/Models/chi2_rank.csv', index=False)
    only_binary_lenses = chi2_df[chi2_df['model'].str.contains('L')]
    only_binary_lenses.to_csv(folder_path + '/Models/chi2_rank_binary_lenses.csv', index=False)
    only_LS = chi2_df[chi2_df['model'].str.contains('LS')]
    only_LX = chi2_df[chi2_df['model'].str.contains('LX')]
    only_LO = chi2_df[chi2_df['model'].str.contains('LO')]
    top_1_of_each = pd.concat([only_LS.iloc[[0]], only_LX.iloc[[0]], only_LO.iloc[[0]]])
    top_1_of_each.to_csv(folder_path + '/Models/chi2_top1_of_each_binary_lens_model.csv', index=False)


def get_summary_of_q_s_chi2_per_event(folder_path, type_of_event):
    """
    This function creates a summary of q and s for the top 1 models + trues values for the given event
    """
    top_1_of_each = pd.read_csv(folder_path + '/Models/chi2_top1_of_each_binary_lens_model.csv')
    data_challenge_lc_number = int(folder_path.split('_')[-1])
    master_path = folder_path.split('/datachallenge_events/')[0]
    true_values_path = f'{master_path}/data'
    the_lightcurve_event = LightcurveEventDataChallenge(data_challenge_lc_number, true_values_path)
    if type_of_event == 'bound_planet':
        true_q = the_lightcurve_event.planet.planet_mass_ratio
        true_s = the_lightcurve_event.planet.planet_separation
    elif type_of_event == 'binary_star':
        true_q = the_lightcurve_event.second_lens.mass_ratio
        true_s = the_lightcurve_event.second_lens.separation
    else:
        raise ValueError(f'type_of_event must be either bound_planet or binary_star. Given {type_of_event}')
    data_to_be_saved = {'data_challenge_lc_number': [data_challenge_lc_number,],
                        'true_q': [true_q,],
                        'true_s': [true_s,],}
    for model_name in top_1_of_each['model']:
        model_type = model_name[0:2]
        model_path = folder_path + '/Models/' + model_name + '.txt'
        model_parameters = ModelResults(model_path,
                                        data_challenge_lc_number=folder_path.split('_')[-1]).model_parameters
        data_to_be_saved[f'{model_type}_q'] = [model_parameters.mass_ratio,]
        data_to_be_saved[f'{model_type}_q_err'] = [model_parameters.mass_ratio_error,]
        data_to_be_saved[f'{model_type}_s'] = [model_parameters.separation,]
        data_to_be_saved[f'{model_type}_s_err'] = [model_parameters.separation_error,]
        data_to_be_saved[f'{model_type}_chi2'] = [model_parameters.chi2,]
    event_summary = pd.DataFrame(data_to_be_saved)
    event_summary.to_csv(folder_path + '/Models/event_summary_q_s.csv', index=False)
    return event_summary


def event_summary_q_s_wrapper(root_path_, list_of_events, type_of_event):
    """
    This function wraps up all the results for the given type of events, and produce a .csv with all of them
    """
    df_general = []
    for event_number_ in list_of_events:
        df_for_one_event = pd.read_csv(f'{root_path_}/event_{event_number_:03}/Models/event_summary_q_s.csv')
        df_general.append(df_for_one_event)
    # Concatenate all the dataframes
    all_q_s_wrapper_df = pd.concat(df_general)
    all_q_s_wrapper_df.to_csv(f'{root_path_}/all_{type_of_event}_q_s.csv', index=False)
    return all_q_s_wrapper_df

#####
def get_summary_of_piN_piE_chi2_per_event(folder_path, type_of_event):
    """
    This function creates a summary of q and s for the top 1 models + trues values for the given event
    """
    top_1_of_each = pd.read_csv(folder_path + '/Models/chi2_top1_of_each_binary_lens_model.csv')
    data_challenge_lc_number = int(folder_path.split('_')[-1])
    master_path = folder_path.split('/datachallenge_events/')[0]
    true_values_path = f'{master_path}/data'
    the_lightcurve_event = LightcurveEventDataChallenge(data_challenge_lc_number, true_values_path)
    if type_of_event == 'bound_planet':
        true_piN = the_lightcurve_event.planet.planet_mass_ratio
        true_piE = the_lightcurve_event.planet.planet_separation
    elif type_of_event == 'binary_star':
        true_q = the_lightcurve_event.second_lens.mass_ratio
        true_s = the_lightcurve_event.second_lens.separation
    else:
        raise ValueError(f'type_of_event must be either bound_planet or binary_star. Given {type_of_event}')
    data_to_be_saved = {'data_challenge_lc_number': [data_challenge_lc_number,],
                        'true_q': [true_q,],
                        'true_s': [true_s,],}
    for model_name in top_1_of_each['model']:
        model_type = model_name[0:2]
        model_path = folder_path + '/Models/' + model_name + '.txt'
        model_parameters = ModelResults(model_type, folder_path.split('_')[-1], model_path).model_parameters
        data_to_be_saved[f'{model_type}_q'] = [model_parameters.mass_ratio,]
        data_to_be_saved[f'{model_type}_q_err'] = [model_parameters.mass_ratio_error,]
        data_to_be_saved[f'{model_type}_s'] = [model_parameters.separation,]
        data_to_be_saved[f'{model_type}_s_err'] = [model_parameters.separation_error,]
        data_to_be_saved[f'{model_type}_chi2'] = [model_parameters.chi2,]
    event_summary = pd.DataFrame(data_to_be_saved)
    event_summary.to_csv(folder_path + '/Models/event_summary_q_s.csv', index=False)
    return event_summary


def event_summary_q_s_wrapper(root_path, list_of_events, type_of_event):
    """
    This function wraps up all the results for the given type of events, and produce a .csv with all of them
    """
    df_general = []
    for event_number_ in list_of_events:
        df_for_one_event = pd.read_csv(f'{root_path}/event_{event_number_:03}/Models/event_summary_q_s.csv')
        df_general.append(df_for_one_event)
    # Concatenate all the dataframes
    all_q_s_wrapper_df = pd.concat(df_general)
    all_q_s_wrapper_df.to_csv(f'{root_path}/all_{type_of_event}_q_s.csv', index=False)
    return all_q_s_wrapper_df
#####3



if __name__ == '__main__':
    # print(chi2_getter('/Users/sishitan/Documents/Scripts/RTModel_project/RTModel/datachallenge_events/event_004/Models/BS0004-1.txt'))
    list_of_bound_planet_events = [4, 8, 12, 25, 32, 40, 47, 50, 53, 62, 66, 69, 74, 78, 81, 92, 95, 99, 100, 103,
                                   107, 124, 128, 131, 139, 152, 163, 186, 193, 194, 199, 208, 214, 217, 218, 223,
                                   226, 227, 250, 253, 258, 267, 289]
    # event_number = list_of_bound_planet_events[0]
    # folder_path_ = (f'/Users/sishitan/Documents/Scripts/RTModel_project/'
    #                 f'RTModel/datachallenge_events/event_{event_number:03}')
    # models_per_chi2_rank(folder_path_)
    # get_summary_of_q_s_chi2_per_event(folder_path_)
    root_path = '/local/data/emussd1/greg_shared/rtmodel_effort/datachallenge/datachallenge_events/'
    # root_path = '/Users/sishitan/Documents/Scripts/RTModel_project/RTModel/datachallenge_events/'

    for event_number in list_of_bound_planet_events:
        folder_path_ = f'{root_path}event_{event_number:03}'
        models_per_chi2_rank(folder_path_)
        get_summary_of_q_s_chi2_per_event(folder_path_, 'bound_planet')

    event_summary_q_s_wrapper(root_path, list_of_bound_planet_events, 'bound_planet')



from jasmine import ModelResults
import pandas as pd


def get_summary_of_parallax_per_event(folder_path):
    """
    This function creates a summary of piN and piE for the top 1 models + trues values for the given event
    """
    top_1_of_each = pd.read_csv(folder_path + '/Models/chi2_top1_of_each_binary_lens_model.csv')
    data_challenge_lc_number = int(folder_path.split('_')[-1])
    data_to_be_saved = {'data_challenge_lc_number': [data_challenge_lc_number,]}
    for model_name in top_1_of_each['model']:
        model_type = model_name[0:2]
        if model_type == 'LO' or model_type == 'LX':
            model_path = folder_path + '/Models/' + model_name + '.txt'
            model_parameters = ModelResults(model_path,
                                            data_challenge_lc_number=folder_path.split('_')[-1]).model_parameters
            data_to_be_saved[f'{model_type}_piN'] = [model_parameters.piN,]
            data_to_be_saved[f'{model_type}_piN_error'] = [model_parameters.piN_error,]
            data_to_be_saved[f'{model_type}_piE'] = [model_parameters.piE,]
            data_to_be_saved[f'{model_type}_piE_error'] = [model_parameters.piE_error,]
            data_to_be_saved[f'{model_type}_chi2'] = [model_parameters.chi2,]
    event_summary = pd.DataFrame(data_to_be_saved)
    event_summary.to_csv(folder_path + '/Models/event_summary_parallax.csv', index=False)
    return event_summary


def event_summary_parallax_wrapper(root_path_, list_of_events, type_of_event):
    """
    This function wraps up all the results for the given type of events, and produce a .csv with all of them
    """
    df_general = []
    for event_number_ in list_of_events:
        df_for_one_event = pd.read_csv(f'{root_path_}/event_{event_number_:03}/Models/event_summary_parallax.csv')
        df_general.append(df_for_one_event)
    # Concatenate all the dataframes
    all_q_s_wrapper_df = pd.concat(df_general)
    all_q_s_wrapper_df.to_csv(f'{root_path_}/all_{type_of_event}_parallax.csv', index=False)
    return all_q_s_wrapper_df


if __name__ == '__main__':
    list_of_bound_planet_events = [4, 8, 12, 25, 32, 40, 47, 50, 53, 62, 66, 69, 74, 78, 81, 92, 95, 99, 100, 103,
                                   107, 124, 128, 131, 139, 152, 163, 186, 193, 194, 199, 208, 214, 217, 218, 223,
                                   226, 227, 250, 253, 258, 267, 289]

    root_path = '/local/data/emussd1/greg_shared/rtmodel_effort/datachallenge/datachallenge_events/'

    for event_number in list_of_bound_planet_events:
        folder_path_ = f'{root_path}event_{event_number:03}'
        get_summary_of_parallax_per_event(folder_path_, 'bound_planet')

    event_summary_parallax_wrapper(root_path, list_of_bound_planet_events, 'bound_planet')


import VBBinaryLensing
from dataclasses import dataclass
import numpy as np
import pandas as pd

@dataclass
class VBBandRTModelTemplateForBinaryLightCurve():
    """
    Generate a light curve based on a line of RTModel template
    """
    template_line: int
    separation_s: float
    mass_ratio_q: float
    impact_parameter_u0: float
    angle_alpha: float
    source_radius_rho: float
    pre_calculated_peak_tp1: float
    pre_calculated_peak_tp2: float
    einstein_time_tE: float
    peak_time_t0: float

    def __init__(self, *, template_line, path_to_template,
                 input_tE, input_t0, cadence_in_days):

        vbb = VBBinaryLensing.VBBinaryLensing()
        self.path_to_template = path_to_template
        line_information = read_template(path_to_template).loc[template_line - 2]
        self.separation_s = line_information['s']
        self.mass_ratio_q = line_information['q']
        self.impact_parameter_u0 = line_information['u0']
        self.angle_alpha = line_information['alpha']
        self.source_radius_rho = line_information['rho']
        self.einstein_time_tE = input_tE
        self.peak_time_t0 = input_t0
        self.cadence = cadence_in_days
        time_interval = np.arange(start=self.peak_time_t0 - 2 * self.einstein_time_tE,
                                  stop=self.peak_time_t0 + 2 * self.einstein_time_tE,
                                  step=self.cadence)
        pr = [np.log(self.separation_s), np.log(self.mass_ratio_q),
              self.impact_parameter_u0, self.angle_alpha,
              np.log(self.source_radius_rho), np.log(self.einstein_time_tE), self.peak_time_t0]
        self.results = vbb.BinaryLightCurve(pr, time_interval)
        self.times = time_interval
        self.magnification = self.results[0]


def read_template(path_to_template):
    template_df = pd.read_csv(path_to_template,
                              skiprows=1,
                              sep=r'\s+',
                              names=['s', 'q', 'u0', 'alpha', 'rho', 'tp1', 'tp2'],
                              )
    return template_df



import pandas as pd
import numpy as np
import os
from astropy.coordinates import SkyCoord
from astropy.coordinates import solar_system_ephemeris, EarthLocation
from astropy.coordinates import get_body_barycentric
from astropy.time import Time


def creating_ephemerides(*,
                         satellite_folder_path_='/Users/jmbrashe/VBBOrbital/satellitedir',
                         ephemerides_path_='/Users/jmbrashe/VBBOrbital/data-challenge-1'):
    if os.path.isdir(satellite_folder_path_):
        print(f'Directory {satellite_folder_path_} already exists! Continuing to file creation.')
    else:
        os.mkdir(satellite_folder_path_)
        print(f'Folder {satellite_folder_path_} created!')
    solar_system_ephemeris.set('de432s')
    filters = ['W149', 'Z087']
    deldot_str = '{:>11}'.format('0.0000000')  # string of zeros for the delta r column
    names = ['BJD', 'X_EQ', 'Y_EQ', 'Z_EQ', 'X_ECL', 'Y_ECL', 'Z_ECL']  # names of data challenge ephemerides columns
    au_per_km = 1 / 149_597_870.700  # for conversion
    for i in range(len(filters)):
        ephname_ = f'satellite{i + 1}.txt'
        if os.path.exists(f'{satellite_folder_path_}/{ephname_}'):
            raise FileExistsError(f'Ephemerides file {ephname_} already exists in directory {satellite_folder_path_}')
        else:
            eph_df = pd.read_csv(ephemerides_path_ + f'/wfirst_ephemeris_{filters[i]}' + '.txt', sep='\s+', names=names)
            eq_coords = SkyCoord(x=eph_df.iloc[:, 1], y=eph_df.iloc[:, 2], z=eph_df.iloc[:, 3],
                                 unit='au', frame='icrs', obstime='J2000', representation_type='cartesian')
            # Get earth location in cartesian ICRS to calculate actual distance ...
            t = Time(eph_df['BJD'], format='jd')
            eloc = get_body_barycentric('earth', t)
            earth_loc = SkyCoord(eloc, unit='km', frame='icrs', obstime='J2000', representation_type='cartesian')
            earth_loc = earth_loc.to_table().to_pandas()
            earth_loc = earth_loc * au_per_km
            roman_loc = eq_coords.to_table().to_pandas()
            roman_earth_distance = (roman_loc - earth_loc) ** 2
            roman_earth_distance = roman_earth_distance.sum(axis=1)
            roman_earth_distance = np.sqrt(roman_earth_distance)
            eq_coords.representation_type = 'spherical'
            eq_coords = eq_coords.to_table().to_pandas()
            eq_coords['distance'] = roman_earth_distance
            eq_coords['delta_dot'] = np.zeros(eq_coords.shape[0])
            eq_coords['BJD'] = eph_df['BJD']
            eq_coords = eq_coords[['BJD', 'ra', 'dec', 'distance', 'delta_dot']]
            with open(f'{satellite_folder_path_}/{ephname_}', 'w') as f:
                f.write('$$SOE\n')
                for j in range(eq_coords.shape[0]):
                    line = makeline(eq_coords.iloc[j, 0:4], deldot_str)
                    f.write(line)
                f.write('$$EOE')
            print(f'File {ephname_} created in {satellite_folder_path_}.')


# function for rounding a string so that is n characters long.
def roundton(sval, n):
    l_ = list(str(sval))
    if len(l_) <= n:
        s_ = "".join(l_)
        return s_.ljust(n, ' ')
    else:
        if int(l_[n]) >= 5:
            if int(l_[n - 1]) < 9:
                l_[n - 1] = str(int(l_[n - 1]) + 1)
            elif int(l_[n - 1]) == 9:
                j = 1
                while int(l_[n - j]) == 9:
                    l_[n - j] = '0'
                    j += 1
                l_[n - j] = str(int(l_[n - j]) + 1)
            s_ = "".join(l_[0:n])
            return s_
        else:
            s_ = "".join(l_[0:n])
            return s_


# makes one line of the ephemerides file.
def makeline(dfline, deldot):
    bjd_str = '{:0<17}'.format(dfline['BJD'])
    ra_str = str(round(dfline['ra'], 5))
    ra_split = ra_str.split('.')
    ra_split[1] = '{:0<5}'.format(ra_split[1])
    ra_str = ra_split[0] + '.' + ra_split[1]
    ra_str = '{:>13}'.format(ra_str)

    dec_str = str(round(dfline['dec'], 5))
    dec_split = dec_str.split('.')
    dec_split[1] = '{:0<5}'.format(dec_split[1])
    dec_str = dec_split[0] + '.' + dec_split[1]
    dec_str = '{:>9}'.format(dec_str)
    dist_str = roundton(dfline['distance'], 16)
    line = bjd_str + ' ' + ra_str + ' ' + dec_str + ' ' + dist_str + ' ' + deldot + '\n'
    return line


if __name__ == '__main__':
    satellite_folder = '/Users/jmbrashe/VBBOrbital/satellitedir/'
    ephemerides_location = '/Users/jmbrashe/VBBOrbital/data-challenge-1/'
    creating_ephemerides(satellite_folder_path_=satellite_folder, ephemerides_path_=ephemerides_location)



from dataclasses import dataclass
import numpy as np


class ModelResults:
    def __init__(self, file_to_be_read, *, data_challenge_lc_number=None):
        self.model_type = file_to_be_read.split('/')[-1][0:2]
        self.data_challenge_lc_number = data_challenge_lc_number
        if self.model_type == 'PS':
            self.model_extensive_name = 'Single Lens Single Source (1L1S)'
            self.model_parameters = SingleLensSingleSourcePS(file_to_be_read)
        elif self.model_type == 'PX':
            self.model_extensive_name = 'Single Lens Single Source with Parallax (1L1S+plx)'
            self.model_parameters = SingleLensSingleSourceWithParallaxPX(file_to_be_read)
        elif self.model_type == 'BS':
            self.model_extensive_name = 'Single Lens Binary Source (1L2S)'
            self.model_parameters = SingleLensBinarySourceBS(file_to_be_read)
        elif self.model_type == 'BO':
            self.model_extensive_name = 'Single Lens Binary Source with Xallarap (1L2S+xlp)'
            self.model_parameters = SingleLensBinarySourceWithXallarapBO(file_to_be_read)
        elif self.model_type == 'LS':
            self.model_extensive_name = 'Binary Lens Single Source (2L1S)'
            self.model_parameters = BinaryLensSingleSourceLS(file_to_be_read)
        elif self.model_type == 'LX':
            self.model_extensive_name = 'Binary Lens Single Source with Parallax (2L1S+plx)'
            self.model_parameters = BinaryLensSingleSourceWithParallaxLX(file_to_be_read)
        elif self.model_type == 'LO':
            self.model_extensive_name = 'Binary Lens Single Source with Parallax and Orbital Motion (2L1S+plx+OM)'
            self.model_parameters = BinaryLensSingleSourceWithOrbitalMotionLO(file_to_be_read)
        else:
            raise ValueError(f"Model type {self.model_type} not recognized.")


@dataclass
class SingleLensSingleSourcePS:
    """
    PS	Single_lens_single_source
    4 parameters
    TODO - not ready. file to be read not specified
    """
    u0: float  # Impact parameter normalized to Einstein angle (internally fit in logarithmic (ln) scale)
    tE: float  # Einstein time in days (internally fit in logarithmic (ln) scale)
    t0: float  # Closest approach time in HJD
    rho: float  # Source radius normalized to Einstein angle (internally fit in logarithmic (ln) scale)
    number_of_parameters = 4

    def __init__(self, file_to_be_read):
        self.u0 = file_to_be_read[0]
        self.tE = file_to_be_read[1]
        self.t0 = file_to_be_read[2]
        self.rho = file_to_be_read[3]


@dataclass
class SingleLensSingleSourceWithParallaxPX:
    """
    PX	Single_lens_single_source with parallax
    6 parameters
    TODO - not ready. file to be read not specified
    """
    u0: float  # Impact parameter normalized to Einstein angle
    tE: float  # Einstein time in days (internally fit in logarithmic (ln) scale)
    t0: float  # Closest approach time in HJD
    rho: float  # Source radius normalized to Einstein angle (internally fit in logarithmic (ln) scale)
    piN: float  # Parallax component along North
    piE: float  # Parallax component along East
    number_of_parameters = 6

    def __init__(self, file_to_be_read):
        self.u0 = file_to_be_read[0]
        self.tE = file_to_be_read[1]
        self.t0 = file_to_be_read[2]
        self.rho = file_to_be_read[3]
        self.piN = file_to_be_read[4]
        self.piE = file_to_be_read[5]


@dataclass
class SingleLensBinarySourceBS:
    """
    BS	Single_lens_binary_source
    7 parameters
    TODO - not ready. file to be read not specified
    """
    tE: float  # Einstein time in days (internally fit in logarithmic (ln) scale)
    flux_ratio: float  # Flux ratio of the secondary to the primary source (internally fit in logarithmic (ln) scale)
    u01: float  # Impact parameter of the primary source
    u02: float  # Impact parameter of the secondary source
    t01: float  # Closest approach time of the primary source
    t02: float  # Closest approach time of the secondary source
    rho1: float  # Source radius for the primary source (internally fit in logarithmic (ln) scale)
    number_of_parameters = 7

    def __init__(self, file_to_be_read):
        self.tE = file_to_be_read[0]
        self.flux_ratio = file_to_be_read[1]
        self.u01 = file_to_be_read[2]
        self.u02 = file_to_be_read[3]
        self.t01 = file_to_be_read[4]
        self.t02 = file_to_be_read[5]
        self.rho1 = file_to_be_read[6]


@dataclass
class SingleLensBinarySourceWithXallarapBO:
    """
    BO	Single_lens_binary_source with xallarap
    10 parameters
    TODO - not ready. file to be read not specified
    """
    u01: float  # Impact parameter of the primary source
    t01: float  # Closest approach time of the primary source
    tE: float  # Einstein time in days (internally fit in logarithmic (ln) scale)
    rho1: float  # Source radius for the primary source (internally fit in logarithmic (ln) scale)
    xi1: float  # Xallarap component parallel to the source velocity
    xi2: float  # Xallarap component orthogonal to the source velocity
    omega: float  # Orbital angular velocity in days^-1
    inc: float  # Inclination of the orbital plane in radians
    phi: float  # Phase of the orbit from the passage on the line of nodes
    qs: float  # Mass ratio of the secondary to the primary source
    number_of_parameters = 10

    def __init__(self, file_to_be_read):
        self.u01 = file_to_be_read[0]
        self.t01 = file_to_be_read[1]
        self.tE = file_to_be_read[2]
        self.rho1 = file_to_be_read[3]
        self.xi1 = file_to_be_read[4]
        self.xi2 = file_to_be_read[5]
        self.omega = file_to_be_read[6]
        self.inc = file_to_be_read[7]
        self.phi = file_to_be_read[8]
        self.qs = file_to_be_read[9]


@dataclass
class BinaryLensSingleSourceLS:
    """
    LS	Binary_lens_single_source
    7 parameters
    """
    number_of_parameters: int
    separation: float  # Separation between the lenses in Einstein radii (internally fit in logarithmic (ln) scale)
    separation_error: float
    mass_ratio: float  # Mass ratio of the secondary to the primary lens (internally fit in logarithmic (ln) scale)
    mass_ratio_error: float
    u0: float  # Impact parameter normalized to Einstein angle
    u0_error: float
    alpha: float  # Angle between the source velocity and the vector pointing from the secondary to the primary lens
    alpha_error: float
    rho: float  # Source radius normalized to Einstein angle (internally fit in logarithmic (ln) scale)
    rho_error: float
    tE: float  # Einstein time in days (internally fit in logarithmic (ln) scale)
    tE_error: float
    t0: float  # Closest approach time in HJD to the barycenter
    t0_error: float
    chi2: float  # Chi2 value for the fit
    blends: tuple
    sources: tuple
    blending: np.array
    baseline: np.array

    def __init__(self, file_to_be_read):
        self.number_of_parameters = 7
        with open(file_to_be_read, 'r') as f:
            lines = f.readlines()
            parameters = lines[0].split(' ')
            errors = lines[1].split(' ')
            self.separation = float(parameters[0])
            self.separation_error = float(errors[0])
            self.mass_ratio = float(parameters[1])
            self.mass_ratio_error = float(errors[1])
            self.u0 = float(parameters[2])
            self.u0_error = float(errors[2])
            self.alpha = float(parameters[3])
            self.alpha_error = float(errors[3])
            self.rho = float(parameters[4])
            self.rho_error = float(errors[4])
            self.tE = float(parameters[5])
            self.tE_error = float(errors[5])
            self.t0 = float(parameters[6])
            self.t0_error = float(errors[6])
            self.chi2 = float(parameters[-1])
            self.blends = float(parameters[7]), float(parameters[9])
            self.sources = float(parameters[8]), float(parameters[10])
            self.blending = np.array(self.blends) / np.array(self.sources)
            self.baseline = -2.5 * np.log10(np.array(self.blends) + np.array(self.sources))


@dataclass
class BinaryLensSingleSourceWithParallaxLX:
    """
    LX	Binary_lens_single_source with parallax
    9 parameters
    """
    number_of_parameters: int
    separation: float  # Separation between the lenses in Einstein radii (internally fit in logarithmic (ln) scale)
    separation_error: float
    mass_ratio: float  # Mass ratio of the secondary to the primary lens (internally fit in logarithmic (ln) scale)
    mass_ratio_error: float
    u0: float  # Impact parameter normalized to Einstein angle
    u0_error: float
    alpha: float  # Angle between the source velocity and the vector pointing from the secondary to the primary lens
    alpha_error: float
    rho: float  # Source radius normalized to Einstein angle (internally fit in logarithmic (ln) scale)
    rho_error: float
    tE: float  # Einstein time in days (internally fit in logarithmic (ln) scale)
    tE_error: float
    t0: float  # Closest approach time in HJD to the barycenter
    t0_error: float
    piN: float  # Parallax component along North
    piN_error: float
    piE: float  # Parallax component along East
    piE_error: float
    chi2: float  # Chi2 value for the fit
    blends: tuple
    sources: tuple
    blending: np.array
    baseline: np.array

    def __init__(self, file_to_be_read):
        self.number_of_parameters = 9
        with open(file_to_be_read, 'r') as f:
            lines = f.readlines()
            parameters = lines[0].split(' ')
            errors = lines[1].split(' ')
            self.separation = float(parameters[0])
            self.separation_error = float(errors[0])
            self.mass_ratio = float(parameters[1])
            self.mass_ratio_error = float(errors[1])
            self.u0 = float(parameters[2])
            self.u0_error = float(errors[2])
            self.alpha = float(parameters[3])
            self.alpha_error = float(errors[3])
            self.rho = float(parameters[4])
            self.rho_error = float(errors[4])
            self.tE = float(parameters[5])
            self.tE_error = float(errors[5])
            self.t0 = float(parameters[6])
            self.t0_error = float(errors[6])
            self.piN = float(parameters[7])
            self.piN_error = float(errors[7])
            self.piE = float(parameters[8])
            self.piE_error = float(errors[8])
            self.chi2 = float(parameters[-1])
            self.blends = float(parameters[9]), float(parameters[11])
            self.sources = float(parameters[10]), float(parameters[12])
            self.blending = np.array(self.blends) / np.array(self.sources)
            self.baseline = -2.5 * np.log10(np.array(self.blends) + np.array(self.sources))


@dataclass
class BinaryLensSingleSourceWithOrbitalMotionLO:
    """
    LO	Binary_lens_single_source with orbital motion
    12 parameters
    """
    number_of_parameters: int
    separation: float  # Separation between the lenses in Einstein radii (internally fit in logarithmic (ln) scale)
    separation_error: float
    mass_ratio: float  # Mass ratio of the secondary to the primary lens (internally fit in logarithmic (ln) scale)
    mass_ratio_error: float
    u0: float  # Impact parameter normalized to Einstein angle
    u0_error: float
    alpha: float  # Angle between the source velocity and the vector pointing from the secondary to the primary lens
    alpha_error: float
    rho: float  # Source radius normalized to Einstein angle (internally fit in logarithmic (ln) scale)
    rho_error: float
    tE: float  # Einstein time in days (internally fit in logarithmic (ln) scale)
    tE_error: float
    t0: float  # Closest approach time in HJD to the barycenter
    t0_error: float
    piN: float  # Parallax component along North
    piN_error: float
    piE: float  # Parallax component along East
    piE_error: float
    gamma1: float  # Angular velocity parallel to the lens axis
    gamma1_error: float
    gamma2: float  # Angular velocity perpendicular to the lens axis
    gamma2_error: float
    gammaz: float  # Angular velocity along the line of sight
    gammaz_error: float
    chi2: float  # Chi2 value for the fit

    def __init__(self, file_to_be_read):
        self.number_of_parameters = 12
        with open(file_to_be_read, 'r') as f:
            lines = f.readlines()
            parameters = lines[0].split(' ')
            errors = lines[1].split(' ')
            self.separation = float(parameters[0])
            self.separation_error = float(errors[0])
            self.mass_ratio = float(parameters[1])
            self.mass_ratio_error = float(errors[1])
            self.u0 = float(parameters[2])
            self.u0_error = float(errors[2])
            self.alpha = float(parameters[3])
            self.alpha_error = float(errors[3])
            self.rho = float(parameters[4])
            self.rho_error = float(errors[4])
            self.tE = float(parameters[5])
            self.tE_error = float(errors[5])
            self.t0 = float(parameters[6])
            self.t0_error = float(errors[6])
            self.piN = float(parameters[7])
            self.piN_error = float(errors[7])
            self.piE = float(parameters[8])
            self.piE_error = float(errors[8])
            self.gamma1 = float(parameters[9])
            self.gamma1_error = float(errors[9])
            self.gamma2 = float(parameters[10])
            self.gamma2_error = float(errors[10])
            self.gammaz = float(parameters[11])
            self.gammaz_error = float(errors[11])
            self.chi2 = float(parameters[-1])
            self.blends = float(parameters[12]), float(parameters[14])
            self.sources = float(parameters[13]), float(parameters[15])
            self.blending = np.array(self.blends) / np.array(self.sources)
            self.baseline = -2.5 * np.log10(np.array(self.blends) + np.array(self.sources))



from setuptools import setup

setup(name='jasmine-astro',
      version='0.1.4',
      description='JASMINE: Joint Analysis of Simulations for Microlensing INterest Events',
      url='https://github.com/stelais/jasmine',
      author='Stela IS',
      author_email='stela.ishitanisilva@nasa.gov',
      license='MIT',
      packages=['jasmine',
                'jasmine.classes_and_files_reader',
                'jasmine.files_organizer',
                'jasmine.investigator',
                'jasmine.lightcurve_signal_generator'],
      platforms="Mac OS 14.5",
      keywords=['Astronomy', 'Microlensing', 'Science'],
      zip_safe=False,
      install_requires=["pandas==2.1.4",
                        "jupyter==1.0.0",
                        "notebook==7.1.3",
                        "ipykernel==6.29.4",
                        "matplotlib==3.8.4",
                        "astropy==6.1.0",
                        "jplephem==2.22",
                        "bokeh==3.4.1",
                        "VBBinaryLensing==3.7.0",
                        ])
